package com.javaranch.pwcij;

public class AccountContract implements Account {
    private Account m_impl;

    public AccountContract(Account impl) {
        m_impl = impl;
    }

    public void credit(double amount) {
        double balance = getBalance();
        assert amount > 0 : "Credit amount nonpositive";
        m_impl.credit(amount);
        assert getBalance() == balance + amount : "Computed balance incorrect";
    }

    public void debit(double amount) {
        double balance = getBalance();
        assert amount > 0 : "Debit amount nonpositive";
        assert amount <= balance : "Debit amount would overdraw";
        m_impl.debit(amount);
        assert getBalance() == balance - amount : "Computed balance incorrect";
    }

    public String getAccountNumber() {
        String result = m_impl.getAccountNumber();
        assert result != null : "Account number is null";
        assert result.length() > 0 : "Account number is empty";
        return result;
    }

    public double getBalance() {
        double result = m_impl.getBalance();
        assert result >= 0 : "Balance has gone negative";
        return result;
    }

    public String getCustomerName() {
        String result = m_impl.getCustomerName();
        assert result != null : "Customer name is null";
        assert result.length() > 0 : "Customer name is empty";
        return result;
    }
}
